/*
 * Copyright 2018 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.verilog.parser;

import ru.ispras.fortress.util.InvariantChecks;
import ru.ispras.verilog.parser.elaborator.VerilogDesign;

/**
 * {@link VerilogDesignBackend} is a basic class for design-level back-ends.
 * 
 * <p>The {@code start} method should be override in a subclass.</p>
 * 
 * @author <a href="mailto:kamkin@ispras.ru">Alexander Kamkin</a>
 */
public abstract class VerilogDesignBackend {
  /** Back-end name. */
  private final String name;

  /**
   * Creates a back-end.
   * 
   * @param name the back-end name.
   */
  protected VerilogDesignBackend(final String name) {
    InvariantChecks.checkNotNull(name);
    this.name = name;
  }

  /**
   * Returns the name of the back-end.
   * 
   * @return the back-end name.
   */
  public final String getName() {
    return name;
  }

  /**
   * Processes the elaborated design.
   * 
   * @param design the elaborated design.
   */
  public abstract void start(final VerilogDesign design);
}
