/*
 * Copyright 2013-2018 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.verilog.parser.core;

/**
 * {@link NodeVisitor} is a basic node visiting interface.
 * 
 * @author <a href="mailto:kamkin@ispras.ru">Alexander Kamkin</a>
 */
public interface NodeVisitor<Tag extends Enum<Tag>> {

  /** {@link Result} contains possible visitor results. */
  public enum Result {
    /**
     * Continue traversal.
     * 
     * <p>This result can be return by the {@code onBegin} and {@code onEnd} handlers.</p>
     */
    OK,

    /**
     * Do not traverse the children subtrees.
     * 
     * <p>This result can be returned by the {@code onEnd} handler only.</p>
     */
    PRUNE,

    /**
     * Repeat traversal of the node.
     * 
     * <p>This result can be returned by the {@code onEnd} handler only.</p>
     */
    REPEAT,

    /**
     * Stop traversal.
     * 
     * <p>This result can be return by the {@code onBegin} and {@code onEnd} handlers.</p>
     */
    ABORT
  }

  /**
   * Is called before traversing the child nodes.
   * 
   * @param node the visiting node.
   * @return the visiting result.
   */
  public Result onBegin(final AbstractNode<Tag> node);

  /**
   * Is called when all of the child nodes have been traversed.
   * 
   * @param node the visiting node.
   * @return the visiting result.
   */
  public Result onEnd(final AbstractNode<Tag> node);
}
