/*
 * Copyright 2012-2018 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.verilog.parser.model;

import java.util.EnumSet;

/**
 * {@link VerilogTable} represents UDP tables.
 * 
 * @author <a href="mailto:kamkin@ispras.ru">Alexander Kamkin</a>
 */
public final class VerilogTable extends VerilogNode {
  public static final Tag TAG = Tag.TABLE;
  public static final EnumSet<Tag> TAGS_CHILDREN = EnumSet.of(Tag.TABLE_ENTRY);

  /**
   * Creates a UDP table.
   * 
   * @param parent the parent node.
   */
  public VerilogTable(final VerilogNode parent) {
    super(TAG, TAGS_CHILDREN, NodeKind.CLASS, parent);
  }

  /**
   * Creates a copy of the UDP table.
   * 
   * @param other the UDP table to be copied.
   * @param parent the parent node.
   */
  public VerilogTable(final VerilogTable other, final VerilogNode parent) {
    super(other, parent);
  }

  @Override
  public VerilogTable clone() {
    return new VerilogTable(this, getParentNode());
  }
}
