/*
 * Copyright 2018 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.verilog.parser;

import org.junit.Test;

import java.io.File;
import java.nio.file.Paths;

/**
 * VCEGAR benchmark jUnit test cases.
 * @author <a href="mailto:smolov@ispras.ru">Sergey Smolov</a>
 */
public class VerilogVcegarTestCase extends VerilogBenchmarkTest {

  /* AR */

  @Test
  public void runTest_AR_example() {
    runTest("main", Paths.get("AR", "example.v").toString());
  }

  /* cache_coherence */

  @Test
  public void runTest_cache_coherence_three_processor_bin_2() {
    runTest("main", Paths.get("cache_coherence", "three_processor_bin_2.v").toString());
  }

  @Test
  public void runTest_cache_coherence_two_processor_bin_2e() {
    runTest("main", Paths.get("cache_coherence", "two_processor_bin_2.v").toString());
  }

  /* ethernet */

  @Test
  public void runTest_ethernet_ethernet_define_112_vcegar() {
    runTest("main", Paths.get("ethernet", "ethernet.define.112_vcegar.v").toString());
  }

  /* ipbdb */

  @Test
  public void runTest_ipbdp_ipbdp_hier() {
    runTest("IPBDP_hier", Paths.get("ipbdp", "ipbdp_hier.v").toString());
  }

  @Test
  public void runTest_ipbdp_old() {
    runTest("IPBDP_hier", Paths.get("ipbdp", "old.v").toString());
  }

  /* itc99_b12 */

  @Test
  public void runTest_itc99_b12_b12_1() {
    runTest("main", Paths.get("itc99_b12", "b12_1.v").toString());
  }

  /* itc99_b13 */

  @Test
  public void runTest_itc99_b13_b13_1() {
    runTest("main", Paths.get("itc99_b13", "b13_1.v").toString());
  }

  /* miim */

  @Test
  public void runTest_miim_vMiim() {
    runTest("main", Paths.get("miim", "vMiim.v").toString());
  }

  /* mpeg */

  @Test
  public void runTest_mpeg_mpeg_1_main() {
    runTest("main", Paths.get("mpeg", "mpeg_1.v").toString());
  }

  @Test
  public void runTest_mpeg_mpeg_1_input_buffer() {
    runTest("input_buffer", Paths.get("mpeg", "mpeg_1.v").toString());
  }

  @Test
  public void runTest_mpeg_mpeg_1_counter() {
    runTest("counter", Paths.get("mpeg", "mpeg_1.v").toString());
  }

  @Test
  public void runTest_mpeg_mpeg_1_fsm_for_pack_start() {
    runTest("fsm_for_pack_start", Paths.get("mpeg", "mpeg_1.v").toString());
  }

  @Test
  public void runTest_mpeg_mpeg_1_parse_pack_header() {
    runTest("parse_pack_header", Paths.get("mpeg", "mpeg_1.v").toString());
  }

  @Test
  public void runTest_mpeg_mpeg_1_fsm_for_syshead_start() {
    runTest("fsm_for_syshead_start", Paths.get("mpeg", "mpeg_1.v").toString());
  }

  @Test
  public void runTest_mpeg_mpeg_1_parse_system_header() {
    runTest("parse_system_header", Paths.get("mpeg", "mpeg_1.v").toString());
  }

  @Test
  public void runTest_mpeg_mpeg_1_read_packet() {
    runTest("read_packet", Paths.get("mpeg", "mpeg_1.v").toString());
  }

  @Test
  public void runTest_mpeg_mpeg_simplified() {
    runTest("main", Paths.get("mpeg", "mpeg_simplified.v").toString());
  }

  @Test
  public void runTest_mpeg_mpeg_simplified1() {
    runTest("main", Paths.get("mpeg", "mpeg_simplified1.v").toString());
  }

  /* pi_bus */

  @Test
  public void runTest_pi_bus_main_1() {
    runTest("main", Paths.get("pi_bus", "main_1.v").toString());
  }

  /* pj_icram */

  @Test
  public void runTest_pj_icram_icram_1() {
    runTest("main", Paths.get("pj_icram", "icram_1.v").toString());
  }

  @Test
  public void runTest_pj_icram_icram_2() {
    runTest("main", Paths.get("pj_icram", "icram_2.v").toString());
  }

  @Test
  public void runTest_pj_icram_icram_3() {
    runTest("main", Paths.get("pj_icram", "icram_3.v").toString());
  }

  @Test
  public void runTest_pj_icram_icram_4() {
    runTest("main", Paths.get("pj_icram", "icram_4.v").toString());
  }

  @Test
  public void runTest_pj_icram_icram_5() {
    runTest("main", Paths.get("pj_icram", "icram_5.v").toString());
  }

  /* pj_icu */

  @Test
  public void runTest_pj_icu_icctl1() {
    runTest("icctl", Paths.get("pj_icu", "icctl1.v").toString());
  }

  /* sdlx */

  @Test
  public void runTest_sdlx_control1() {
    runTest("main", Paths.get("sdlx", "control1.v").toString());
  }

  /* small/bit-vector */

  @Test
  public void runTest_small_bit_vector_bit_vector() {
    runTest("main", Paths.get("small", "bit-vector", "bit-vector.v").toString());
  }

  /* small/bug-1 */

  @Test
  public void runTest_small_bug_1_1() {
    runTest("main", Paths.get("small", "bug-1", "1.v").toString());
  }

  /* small/dyn_partition */

  @Test
  public void runTest_small_dyn_partition_dp() {
    runTest("main", Paths.get("small", "dyn_partition", "dp.v").toString());
  }

  /* small/equiv */

  @Test
  public void runTest_small_equiv_equiv() {
    runTest("main", Paths.get("small", "equiv", "equiv.v").toString());
  }

  /* small/pipeline */

  @Test
  public void runTest_small_pipeline_pipeline() {
    runTest("main", Paths.get("small", "pipeline", "pipeline.v").toString());
  }

  /* small/seq */

  @Test
  public void runTest_small_seq_example() {
    runTest("main", Paths.get("small", "seq", "example.v").toString());
  }

  /* small/swap */

  @Test
  public void runTest_small_swap_swap1() {
    runTest("main", Paths.get("small", "swap", "swap1.v").toString());
  }

  @Test
  public void runTest_small_swap_swap2() {
    runTest("main", Paths.get("small", "swap", "swap2.v").toString());
  }

  /* small/synabs */

  @Test
  public void runTest_small_synabs_synabs() {
    runTest("main", Paths.get("small", "synabs", "synabs.v").toString());
  }

  /* small/test */

  @Test
  public void runTest_small_test_synabs() {
    runTest("main", Paths.get("small", "test", "synabs.v").toString());
  }

  /* solver_prob0 */

  @Test
  public void runTest_solver_prob0_solver_prob0() {
    runTest("Mod_7", Paths.get("solver_prob0", "solver_prob0.v").toString());
  }

  /* usb_phy */

  @Test
  public void runTest_usb_phy_usb_phy_1() {
    runTest("main", Paths.get("usb_phy", "usb_phy_1.v").toString());
  }

  /* zaher */

  @Test
  public void runTest_zaher_zdlx_impl_v_for_pred() {
    runTest("main", Paths.get("zaher", "zdlx_impl.v_for_pred.v").toString());
  }

  @Override
  String getTestDirPath() {
    return System.getProperty("user.dir")
        + File.separator
        + Paths.get("src", "test", "verilog", "vcegar-tests").toString()
        + File.separator;
  }
}
