`include "mem_2p.v"

module FIFO36
  #(
    parameter ALMOST_FULL_OFFSET = 0,
    parameter ALMOST_EMPTY_OFFSET = 0,
    parameter FIRST_WORD_FALL_THROUGH = "FALSE",
    parameter DATA_WIDTH = 4,
    parameter EN_SYN = "TRUE",
    parameter DO_REG = 1,
    parameter SIM_MODE = "SAFE"
  )

  (
    DO,
    DOP,
    FULL,
    ALMOSTFULL,
    EMPTY,
    ALMOSTEMPTY,
    WRCOUNT,
    RDCOUNT,
    WRERR,
    RDERR,
    DI,
    DIP,
    WREN,
    RDEN,
    RST,
    WRCLK,
    RDCLK
  );

  localparam ADDR_SIZE = 4;
  localparam DATA_SIZE = 32;

  output [(DATA_SIZE-1):0] DO;
  output DOP;

  output FULL;
  output ALMOSTFULL;

  output EMPTY;
  output ALMOSTEMPTY;

  output WRCOUNT;
  output RDCOUNT;

  output WRERR;
  output RDERR;

  input [(DATA_SIZE-1):0] DI;
  input DIP;

  input WREN;
  input RDEN;

  input RST;

  input RDCLK;
  input WRCLK;

  reg [(DATA_SIZE-1):0] DO; 
  wire DOP;
  reg EMPTY;
  reg RR_FULL;
  reg R_FULL;
  reg [12:0] WRCOUNT;
  reg [12:0] RDCOUNT;

  assign DOP = DIP;

  assign FULL = (RR_FULL | R_FULL);

  assign ALMOSTFULL = FULL;
  assign ALMOSTEMPTY = EMPTY;

  assign WRERR = WREN & FULL;
  assign RDERR = RDEN & EMPTY;

  assign CLK = (RDCLK | WRCLK);

  reg [(ADDR_SIZE-1):0] RD_A;   // Read Address
  reg [(ADDR_SIZE-1):0] WR_A;   // Write Address
  reg [(ADDR_SIZE-1):0] RD_A_1; // Read Address + 1
  reg [(ADDR_SIZE-1):0] WR_A_1; // Write Address + 1

  wire [(DATA_SIZE-1):0] DO_tmp;

  reg WREN_tmp;
  reg RDEN_tmp;
  reg [(DATA_SIZE-1):0] DI_tmp;

  mem_2p #(ADDR_SIZE,DATA_SIZE) mem(
    .DO(DO_tmp),
    .RD_A(RD_A),
    .WR_A(WR_A),
    .DI(DI_tmp),
    .RD_CLK(CLK),
    .WR_CLK(CLK),
    .CE_N_RD(~RDEN_tmp),
    .CE_N_WR(~WREN_tmp),
    .OE_N(1'b0)
  );

  always @(posedge CLK) begin
    if(RST) begin
      EMPTY <= 1'b1;
      R_FULL <= 1'b0;
      RR_FULL <= 1'b0;

      RD_A <= {(1 << ADDR_SIZE){1'b1}};
      WR_A <= {(1 << ADDR_SIZE){1'b1}};
      RD_A_1 <= {(1 << ADDR_SIZE){1'b0}};
      WR_A_1 <= {(1 << ADDR_SIZE){1'b0}};

      RDEN_tmp <= 1'b0;
      WREN_tmp <= 1'b0;
    end
  end

  always @(posedge CLK) begin
    DO <= DO_tmp;
    DI_tmp <= DI;
  end

  always @(posedge CLK) begin
    if(RDEN && !EMPTY) begin
      if(!WREN) begin
        if(RD_A_1 == WR_A) begin
          EMPTY <= 1'b1; 
        end
      end

      R_FULL <= 1'b0;
      RD_A <= RD_A_1;
      RD_A_1 <= RD_A_1 + 1;

      RDEN_tmp <= RDEN;

      RDCOUNT <= RDCOUNT + 1;
    end else begin
      RDEN_tmp <= 1'b0;
    end

    if(WREN && !R_FULL) begin
      if(!RDEN) begin
        if(WR_A_1 == RD_A) begin
          R_FULL <= 1'b1;
        end
      end

      EMPTY <= 1'b0;
      WR_A <= WR_A_1;
      WR_A_1 <= WR_A_1 + 1;

      WREN_tmp <= WREN;

      WRCOUNT <= WRCOUNT + 1;
    end else begin
      WREN_tmp <= 1'b0;
    end

    RR_FULL <= R_FULL;
end

endmodule
