`timescale 1ns/1ps

module ram_testbench;
  reg         clk;
  reg         rst;
  reg         val_rd;
  reg         val_wr;
  reg   [1:0] addr_in;
  reg  [31:0] data_in;
  wire        val_out;
  wire [31:0] data_out;
  wire        is_ready;

  ram target
  (
    .clk(clk),
    .rst(rst),
    .val_rd(val_rd),
    .val_wr(val_wr),
    .addr_in(addr_in),
    .data_in(data_in),
    .val_out(val_out),
    .data_out(data_out),
    .is_ready(is_ready)
  );

  initial rst = 0;
  initial clk = 0;
  always  clk = #10 ~clk;

  initial begin
    $dumpvars(4, ram_testbench);
    $display("testing ram...");

    val_rd  = 1'b0;
    val_wr  = 1'b0;
    addr_in = 2'h0;
    data_in = 32'h0000_0000;

    #100;

    $display("resetting ram...");
    rst = 1; #10; rst = 0;

    $display("is_ready=%b", is_ready);

    #100;

    $display("writing to memory...");
    @(negedge clk);
    val_wr  <= 1;
    addr_in <= 2'h2;
    data_in <= 32'h1234_5678;
    @(posedge clk);
    @(negedge clk);
    val_wr  <= 0;
    addr_in <= 2'h0;
    data_in <= 32'h0000_0000;

    #100;

    @(negedge clk);
    val_rd  <= 1;
    addr_in <= 2'h2;
    @(posedge clk);
    @(negedge clk);
    val_rd  <= 0;
    addr_in <= 2'h0;
    
    wait (val_out);

    $display("reading from memory %h...", data_out);

    #100;

    $display("done");
    $finish();
  end
endmodule /* ram_testbench */

